"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.meta = undefined;

var _assign = require("babel-runtime/core-js/object/assign");

var _assign2 = _interopRequireDefault(_assign);

var _promise = require("babel-runtime/core-js/promise");

var _promise2 = _interopRequireDefault(_promise);

var _regenerator = require("babel-runtime/regenerator");

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require("babel-runtime/helpers/asyncToGenerator");

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

exports.default = NetlifyCmsModule;

var _path = require("path");

var _fsExtra = require("fs-extra");

var _lodash = require("lodash");

var _lodash2 = _interopRequireDefault(_lodash);

var _nuxt = require("nuxt");

var _chokidar = require("chokidar");

var _chokidar2 = _interopRequireDefault(_chokidar);

var _pify = require("pify");

var _pify2 = _interopRequireDefault(_pify);

var _webpack = require("webpack");

var _webpack2 = _interopRequireDefault(_webpack);

var _webpackDevMiddleware = require("webpack-dev-middleware");

var _webpackDevMiddleware2 = _interopRequireDefault(_webpackDevMiddleware);

var _webpackHotMiddleware = require("webpack-hot-middleware");

var _webpackHotMiddleware2 = _interopRequireDefault(_webpackHotMiddleware);

var _serveStatic = require("serve-static");

var _serveStatic2 = _interopRequireDefault(_serveStatic);

var _debug = require("debug");

var _debug2 = _interopRequireDefault(_debug);

var _package = require("../package.json");

var _package2 = _interopRequireDefault(_package);

var _configManager = require("./configManager");

var _configManager2 = _interopRequireDefault(_configManager);

var _webpack3 = require("./webpack.config");

var _webpack4 = _interopRequireDefault(_webpack3);

var _yaml = require("./utils/yaml");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* covered by nuxt */
var debug = (0, _debug2.default)("nuxt:netlify-cms"); /* eslint-disable import/no-extraneous-dependencies */


var WEBPACK_CLIENT_COMPILER_NAME = "client";
var WEBPACK_NETLIFY_COMPILER_NAME = "netlify-cms";
var NETLIFY_CONFIG_FILE_NAME = "config.yml";

function NetlifyCmsModule(moduleOptions) {
  var _this = this;

  var configManager = new _configManager2.default(this.options, moduleOptions);
  var config = configManager.config;

  // This will be called once when builder started
  this.nuxt.hook("build:before", function (builder) {
    // This will be run just before webpack compiler starts
    _this.nuxt.hook("build:compile", function (_ref) {
      var name = _ref.name;

      if (name !== WEBPACK_CLIENT_COMPILER_NAME) {
        return;
      }
      var webpackConfig = (0, _webpack4.default)(WEBPACK_NETLIFY_COMPILER_NAME, _this.options, config);

      webpackConfig.plugins.push({
        apply: function apply(compiler) {
          compiler.plugin("emit", function (compilation, cb) {
            var netlifyConfigYAML = (0, _yaml.toYAML)(configManager.cmsConfig);
            compilation.assets[NETLIFY_CONFIG_FILE_NAME] = {
              source: function source() {
                return netlifyConfigYAML;
              },
              size: function size() {
                return netlifyConfigYAML.length;
              }
            };
            cb();
          });
        }
      });

      var netlifyCompiler = (0, _webpack2.default)(webpackConfig);

      // This will be run just after webpack compiler ends
      netlifyCompiler.plugin("done", function () {
        var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(stats) {
          return _regenerator2.default.wrap(function _callee$(_context) {
            while (1) {
              switch (_context.prev = _context.next) {
                case 0:
                  if (!stats.hasErrors()) {
                    _context.next = 2;
                    break;
                  }

                  return _context.abrupt("return");

                case 2:
                  debug("Bundle built!");

                case 3:
                case "end":
                  return _context.stop();
              }
            }
          }, _callee, _this);
        }));

        return function (_x) {
          return _ref2.apply(this, arguments);
        };
      }());

      // in development
      if (_this.options.dev) {
        // Use shared filesystem and cache
        netlifyCompiler.outputFileSystem = builder.mfs;
        // Show a message inside console when the build is ready
        _this.nuxt.hook("build:compiled", (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2() {
          return _regenerator2.default.wrap(function _callee2$(_context2) {
            while (1) {
              switch (_context2.prev = _context2.next) {
                case 0:
                  debug("Serving on: " + config.adminPath);

                case 1:
                case "end":
                  return _context2.stop();
              }
            }
          }, _callee2, _this);
        })));

        // Create webpack dev middleware
        var netlifyWebpackDevMiddleware = (0, _pify2.default)((0, _webpackDevMiddleware2.default)(netlifyCompiler, {
          publicPath: "/",
          stats: builder.webpackStats,
          noInfo: true,
          quiet: true,
          watchOptions: _this.options.watchers.webpack
        }));

        // Create webpack hot middleware
        var netlifyWebpackHotMiddleware = (0, _pify2.default)((0, _webpackHotMiddleware2.default)(netlifyCompiler, {
          log: false,
          heartbeat: 1000
        }));

        // Inject to renderer instance
        if (builder.nuxt.renderer) {
          builder.nuxt.renderer.netlifyWebpackDevMiddleware = netlifyWebpackDevMiddleware;
          builder.nuxt.renderer.netlifyWebpackHotMiddleware = netlifyWebpackHotMiddleware;
        }

        // Stop webpack middleware on nuxt.close()
        _this.nuxt.hook("close", (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3() {
          return _regenerator2.default.wrap(function _callee3$(_context3) {
            while (1) {
              switch (_context3.prev = _context3.next) {
                case 0:
                  _context3.next = 2;
                  return _this.nuxt.renderer.netlifyWebpackDevMiddleware.close();

                case 2:
                case "end":
                  return _context3.stop();
              }
            }
          }, _callee3, _this);
        })));
      } else {
        // Only run the compiler in production,
        // in dev build is started by dev-middleware hooked to client webpack compiler
        _this.nuxt.hook("build:done", (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee4() {
          return _regenerator2.default.wrap(function _callee4$(_context4) {
            while (1) {
              switch (_context4.prev = _context4.next) {
                case 0:
                  _context4.next = 2;
                  return new _promise2.default(function (resolve, reject) {
                    netlifyCompiler.run(function (err, stats) {
                      /* istanbul ignore next */
                      if (err) {
                        return reject(err);
                      } else if (stats.hasErrors()) {
                        if (_this.options.test) {
                          err = stats.toString(_this.options.build.stats);
                        }

                        return reject(err);
                      }

                      resolve();
                    });
                  });

                case 2:
                case "end":
                  return _context4.stop();
              }
            }
          }, _callee4, _this);
        })));
      }
    });
  });

  // Serve netlify CMS
  if (this.options.dev) {
    // Insert webpackDevMiddleware to serve netlify CMS in development
    this.addServerMiddleware({
      path: config.adminPath,
      handler: function () {
        var _ref6 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee5(req, res) {
          return _regenerator2.default.wrap(function _callee5$(_context5) {
            while (1) {
              switch (_context5.prev = _context5.next) {
                case 0:
                  if (!_this.nuxt.renderer.netlifyWebpackDevMiddleware) {
                    _context5.next = 4;
                    break;
                  }

                  debug("requesting url: " + _nuxt.Utils.urlJoin(config.adminPath, req.url));
                  _context5.next = 4;
                  return _this.nuxt.renderer.netlifyWebpackDevMiddleware(req, res);

                case 4:
                  if (!_this.nuxt.renderer.netlifyWebpackHotMiddleware) {
                    _context5.next = 7;
                    break;
                  }

                  _context5.next = 7;
                  return _this.nuxt.renderer.netlifyWebpackHotMiddleware(req, res);

                case 7:
                case "end":
                  return _context5.stop();
              }
            }
          }, _callee5, _this);
        }));

        function handler(_x2, _x3) {
          return _ref6.apply(this, arguments);
        }

        return handler;
      }()
    });

    // Start watching config file
    var patterns = [_nuxt.Utils.r(configManager.cmsConfigFileName)];

    var options = (0, _assign2.default)({}, this.options.watchers.chokidar, {
      ignoreInitial: true
    });

    var refreshFiles = _lodash2.default.debounce(function () {
      configManager.readCmsConfigFile();
      _this.nuxt.renderer.netlifyWebpackDevMiddleware.invalidate();
      _this.nuxt.renderer.netlifyWebpackHotMiddleware.publish({
        action: "reload"
      });
    }, 200);

    // Watch for src Files
    var fileWatcher = _chokidar2.default.watch(patterns, options).on("add", refreshFiles).on("change", refreshFiles).on("unlink", refreshFiles);

    this.nuxt.renderer.netlifyFileWatcher = fileWatcher;

    // Stop watching on nuxt.close()
    this.nuxt.hook("close", function () {
      _this.nuxt.renderer.netlifyFileWatcher.close();
    });
  } else {
    // Statically serve netlify CMS (i.e. .nuxt/dist/admin/) files in production
    this.addServerMiddleware({
      path: config.adminPath,
      handler: (0, _serveStatic2.default)(config.buildDir, {
        maxAge: "1y" // 1 year in production
      })
    });
  }

  // Move cms folder from `dist/_nuxt` folder to `dist/` after nuxt generate
  this.nuxt.hook("generate:distCopied", function () {
    var _ref7 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee6(generator) {
      return _regenerator2.default.wrap(function _callee6$(_context6) {
        while (1) {
          switch (_context6.prev = _context6.next) {
            case 0:
              _context6.next = 2;
              return (0, _fsExtra.move)((0, _path.join)(generator.distNuxtPath, config.adminPath).replace(/\/$/, ""), (0, _path.join)(generator.distPath, config.adminPath).replace(/\/$/, ""));

            case 2:
              debug("Netlify CMS files copied");

            case 3:
            case "end":
              return _context6.stop();
          }
        }
      }, _callee6, _this);
    }));

    return function (_x4) {
      return _ref7.apply(this, arguments);
    };
  }());
}

// REQUIRED if publishing as an NPM package
exports.meta = _package2.default;